<?php

namespace Litecms\Onboardchecklist\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Onboardchecklist\Http\Requests\ChecklistRequest;
use Litecms\Onboardchecklist\Interfaces\ChecklistRepositoryInterface;
use Litecms\Onboardchecklist\Models\Checklist;

/**
 * Resource controller class for checklist.
 */
class ChecklistResourceController extends BaseController
{

    /**
     * Initialize checklist resource controller.
     *
     * @param type ChecklistRepositoryInterface $checklist
     *
     * @return null
     */
    public function __construct(ChecklistRepositoryInterface $checklist)
    {
        parent::__construct();
        $this->repository = $checklist;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Onboardchecklist\Repositories\Criteria\ChecklistResourceCriteria::class);
    }

    /**
     * Display a list of checklist.
     *
     * @return Response
     */
    public function index(ChecklistRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Onboardchecklist\Repositories\Presenter\ChecklistPresenter::class)
                ->$function();
        }

        $checklists = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('onboardchecklist::checklist.names'))
            ->view('onboardchecklist::checklist.index', true)
            ->data(compact('checklists', 'view'))
            ->output();
    }

    /**
     * Display checklist.
     *
     * @param Request $request
     * @param Model   $checklist
     *
     * @return Response
     */
    public function show(ChecklistRequest $request, Checklist $checklist)
    {

        if ($checklist->exists) {
            $view = 'onboardchecklist::checklist.show';
        } else {
            $view = 'onboardchecklist::checklist.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('onboardchecklist::checklist.name'))
            ->data(compact('checklist'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new checklist.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ChecklistRequest $request)
    {

        $checklist = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('onboardchecklist::checklist.name')) 
            ->view('onboardchecklist::checklist.create', true) 
            ->data(compact('checklist'))
            ->output();
    }

    /**
     * Create new checklist.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ChecklistRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $checklist                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('onboardchecklist::checklist.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('onboardchecklist/checklist/' . $checklist->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/onboardchecklist/checklist'))
                ->redirect();
        }

    }

    /**
     * Show checklist for editing.
     *
     * @param Request $request
     * @param Model   $checklist
     *
     * @return Response
     */
    public function edit(ChecklistRequest $request, Checklist $checklist)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('onboardchecklist::checklist.name'))
            ->view('onboardchecklist::checklist.edit', true)
            ->data(compact('checklist'))
            ->output();
    }

    /**
     * Update the checklist.
     *
     * @param Request $request
     * @param Model   $checklist
     *
     * @return Response
     */
    public function update(ChecklistRequest $request, Checklist $checklist)
    {
        try {
            $attributes = $request->all();

            $checklist->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('onboardchecklist::checklist.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('onboardchecklist/checklist/' . $checklist->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('onboardchecklist/checklist/' . $checklist->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the checklist.
     *
     * @param Model   $checklist
     *
     * @return Response
     */
    public function destroy(ChecklistRequest $request, Checklist $checklist)
    {
        try {

            $checklist->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('onboardchecklist::checklist.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('onboardchecklist/checklist/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('onboardchecklist/checklist/' . $checklist->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple checklist.
     *
     * @param Model   $checklist
     *
     * @return Response
     */
    public function delete(ChecklistRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('onboardchecklist::checklist.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('onboardchecklist/checklist'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/onboardchecklist/checklist'))
                ->redirect();
        }

    }

    /**
     * Restore deleted checklists.
     *
     * @param Model   $checklist
     *
     * @return Response
     */
    public function restore(ChecklistRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('onboardchecklist::checklist.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/onboardchecklist/checklist'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/onboardchecklist/checklist/'))
                ->redirect();
        }

    }

}
